#![feature(type_alias_impl_trait)]

use async_bincode::{AsyncBincodeStream, AsyncDestination};
use std::future::Future;
use std::net::SocketAddr;
use std::task::{Context, Poll};
use tokio_tower::multiplex;
use tower_service::Service;

type Transport = AsyncBincodeStream<
    tokio::net::TcpStream,
    (),
    (),
    AsyncDestination,
>;

struct TableEndpoint(SocketAddr);

impl Service<()> for TableEndpoint {
    type Response = multiplex::MultiplexTransport<Transport, ()>;
    type Error = tokio::io::Error;
    // have to repeat types because https://github.com/rust-lang/rust/issues/57807
    type Future = impl Future<
        Output = Result<multiplex::MultiplexTransport<Transport, ()>, tokio::io::Error>,
    >;

    fn poll_ready(&mut self, _: &mut Context<'_>) -> Poll<Result<(), Self::Error>> {
        unimplemented!()
    }

    fn call(&mut self, _: ()) -> Self::Future {
        let f = tokio::net::TcpStream::connect(self.0);
        async move {
            let s = f.await?;
            let s = AsyncBincodeStream::from(s).for_async();
            Ok(multiplex::MultiplexTransport::new(s, ()))
        }
    }
}
